@@ -138,8 +138,7 @@ def add_parameter(key, key_option, value, value_option, stage, input, format, ve
     parameters = []
 
     def validate_command_usage():
-        nonlocal stage, key, value, parameters
-        stage = Stages.normalize(stage)
+        nonlocal key, value, parameters
         validate_not_all_provided([global_scope, project_scope], ["'--global-scope'", "'--project-scope'"])
 
         if input:
@@ -214,8 +213,7 @@ def add_parameter(key, key_option, value, value_option, stage, input, format, ve
 def list_parameter(stage, uninherited, filter, query_all, query, format, verbosity, global_scope, project_scope, config_override, working_dir):
 
     def validate_command_usage():
-        nonlocal stage, query
-        stage = Stages.normalize(stage)
+        nonlocal query
         validate_not_all_provided([global_scope, project_scope], ["'--global-scope'", "'--project-scope'"])
         defaultQuery = '{Parameters: Parameters[*].{Key: Key, Value: Value, Scope: Scope, Origin: Origin}}'
         query = validate_query_argument(query, query_all, defaultQuery)
@@ -251,7 +249,6 @@ def list_parameter(stage, uninherited, filter, query_all, query, format, verbosi
 @click.option('-k', '--key', 'key_option', required=False, metavar='<key>', help=f"{CLI_PARAMETERS_HELP['parameter']['key']}")
 @click.option('-s', '--stage', metavar='<name>[/<number>]', default='default', help=f"{CLI_PARAMETERS_HELP['common']['stage']}")
 @click.option('--revision', required=False, help=f"{CLI_PARAMETERS_HELP['parameter']['revision']}")
-@click.option('--history', required=False, is_flag=True, help=f"{CLI_PARAMETERS_HELP['parameter']['history']}")
 @click.option('-a', '--query-all', required=False, is_flag=True, default=False, show_default=True, help=f"{CLI_PARAMETERS_HELP['common']['query_all']}")
 @click.option('-q', '--query', metavar='<jmespath>', required=False, help=f"{CLI_PARAMETERS_HELP['common']['query']}")
 @click.option('-f', '--format', required=False, type=click.Choice(['json', 'yaml', 'raw', 'csv']), default='json', show_default=True, help=f"{CLI_PARAMETERS_HELP['common']['format']}")
@@ -260,18 +257,13 @@ def list_parameter(stage, uninherited, filter, query_all, query, format, verbosi
 @click.option('--project-scope', required=False, is_flag=True, help=f"{CLI_PARAMETERS_HELP['common']['project_scope']}")
 @click.option('--config-override', 'config_override', metavar='<key>=<value>,...', required=False, help=f"{CLI_PARAMETERS_HELP['common']['config']}")
 @click.option('-w','--working-dir', metavar='<path>', type=click.Path(exists=True, file_okay=False), required=False, help=f"{CLI_PARAMETERS_HELP['common']['working_dir']}")
-def get_parameter(key, key_option, stage, revision, history, query_all, query, format, verbosity, global_scope, project_scope, config_override, working_dir):
+def get_parameter(key, key_option, stage, revision, query_all, query, format, verbosity, global_scope, project_scope, config_override, working_dir):
 
     def validate_command_usage():
-        nonlocal stage, key, query
-        stage = Stages.normalize(stage)
+        nonlocal key, query
         validate_not_all_provided([key, key_option], ["KEY", "'-k' / '--key'"])
         key = validate_at_least_one_provided([key, key_option], ["KEY", "'-k' / '--key'"])
-        validate_not_all_provided([history, revision], ["'--history'", "'--revision'"])
-        if history:
-            defaultQuery = '{Revisions: Revisions[*].{RevisionId: RevisionId, Date: Date, Value: Value, Scope: Scope, Origin: Origin}}'
-        else:
-            defaultQuery = '{Value: Value}'
+        defaultQuery = '{Value: Value}'
         query = validate_query_argument(query, query_all, defaultQuery)
         validate_not_all_provided([global_scope, project_scope], ["'--global-scope'", "'--project-scope'"])
 
@@ -281,10 +273,49 @@ def get_parameter(key, key_option, stage, revision, history, query_all, query, f
         Configs.load(working_dir if working_dir else os.getcwd(),
                         'global' if global_scope else 'project' if project_scope else 'application',
                         config_override)
-        if history:
-            result = Parameters.history(stage, key)
-        else:
-            result = Parameters.get(stage, key, revision)
+        result = Parameters.get(stage, key, revision)
+        output = format_data(result, query, format)
+        if output: Pager.page(output)
+    except DSOException as e:
+        Logger.error(e.message)
+    except Exception as e:
+        msg = getattr(e, 'message', getattr(e, 'msg', str(e)))
+        Logger.critical(msg)
+        if verbosity >= log_levels['full']:
+            raise
+
+###--------------------------------------------------------------------------------------------
+
+@parameter.command('history', context_settings=DEFAULT_CLICK_CONTEXT, short_help=f"{CLI_COMMANDS_SHORT_HELP['parameter']['history']}")
+@command_doc(CLI_COMMANDS_HELP['parameter']['history'])
+@click.argument('key', required=False)
+@click.option('-k', '--key', 'key_option', required=False, metavar='<key>', help=f"{CLI_PARAMETERS_HELP['parameter']['key']}")
+@click.option('-s', '--stage', metavar='<name>[/<number>]', default='default', help=f"{CLI_PARAMETERS_HELP['common']['stage']}")
+@click.option('-a', '--query-all', required=False, is_flag=True, default=False, show_default=True, help=f"{CLI_PARAMETERS_HELP['common']['query_all']}")
+@click.option('-q', '--query', metavar='<jmespath>', required=False, help=f"{CLI_PARAMETERS_HELP['common']['query']}")
+@click.option('-f', '--format', required=False, type=click.Choice(['json', 'yaml', 'raw', 'csv']), default='json', show_default=True, help=f"{CLI_PARAMETERS_HELP['common']['format']}")
+@click.option('-b', '--verbosity', metavar='<number>', required=False, type=RangeParamType(click.INT, minimum=0, maximum=5), default='2', show_default=True, help=f"{CLI_PARAMETERS_HELP['common']['verbosity']}")
+@click.option('--global-scope', required=False, is_flag=True, help=f"{CLI_PARAMETERS_HELP['common']['global_scope']}")
+@click.option('--project-scope', required=False, is_flag=True, help=f"{CLI_PARAMETERS_HELP['common']['project_scope']}")
+@click.option('--config-override', 'config_override', metavar='<key>=<value>,...', required=False, help=f"{CLI_PARAMETERS_HELP['common']['config']}")
+@click.option('-w','--working-dir', metavar='<path>', type=click.Path(exists=True, file_okay=False), required=False, help=f"{CLI_PARAMETERS_HELP['common']['working_dir']}")
+def history_parameter(key, key_option, stage, query_all, query, format, verbosity, global_scope, project_scope, config_override, working_dir):
+
+    def validate_command_usage():
+        nonlocal key, query
+        validate_not_all_provided([key, key_option], ["KEY", "'-k' / '--key'"])
+        key = validate_at_least_one_provided([key, key_option], ["KEY", "'-k' / '--key'"])
+        defaultQuery = '{Revisions: Revisions[*].{RevisionId: RevisionId, Date: Date, Value: Value}}'
+        query = validate_query_argument(query, query_all, defaultQuery)
+        validate_not_all_provided([global_scope, project_scope], ["'--global-scope'", "'--project-scope'"])
+
+    try:
+        Logger.set_verbosity(verbosity)
+        validate_command_usage()
+        Configs.load(working_dir if working_dir else os.getcwd(),
+                        'global' if global_scope else 'project' if project_scope else 'application',
+                        config_override)
+        result = Parameters.history(stage, key)
         output = format_data(result, query, format)
         if output: Pager.page(output)
     except DSOException as e:
@@ -314,7 +345,7 @@ def delete_parameter(key, key_option, input, format, stage, verbosity, global_sc
     parameters = []
 
     def validate_command_usage():
-        nonlocal stage, key, parameters
+        nonlocal key, parameters
         stage = Stages.normalize(stage)
         validate_not_all_provided([global_scope, project_scope], ["'--global-scope'", "'--project-scope'"])
 
@@ -380,8 +411,7 @@ def add_secret(key, key_option, stage, input, format, verbosity, global_scope, p
     secrets = []
 
     def validate_command_usage():
-        nonlocal stage, key, secrets
-        stage = Stages.normalize(stage)
+        nonlocal key, secrets
         validate_not_all_provided([global_scope, project_scope], ["'--global-scope'", "'--project-scope'"])
 
         if input:
@@ -443,7 +473,7 @@ def add_secret(key, key_option, stage, input, format, verbosity, global_scope, p
 @secret.command('list', context_settings=DEFAULT_CLICK_CONTEXT, short_help=f"{CLI_COMMANDS_SHORT_HELP['secret']['list']}")
 @command_doc(CLI_COMMANDS_HELP['secret']['list'])
 @click.option('-s', '--stage', metavar='<name>[/<number>]', default='default', help=f"{CLI_PARAMETERS_HELP['common']['stage']}")
-@click.option('-d', '--decrypt', required=False, is_flag=True, default=False, show_default=True, help=f"{CLI_PARAMETERS_HELP['parameter']['query_values']}")
+@click.option('-d', '--decrypt', required=False, is_flag=True, default=False, show_default=True, help=f"{CLI_PARAMETERS_HELP['secret']['decrypt']}")
 @click.option('-u','--uninherited', 'uninherited', is_flag=True, default=False, help=f"{CLI_PARAMETERS_HELP['secret']['uninherited']}")
 @click.option('--filter', required=False, metavar='<regex>', help=f"{CLI_PARAMETERS_HELP['common']['filter']}")
 @click.option('-a', '--query-all', required=False, is_flag=True, default=False, show_default=True, help=f"{CLI_PARAMETERS_HELP['common']['query_all']}")
@@ -457,8 +487,7 @@ def add_secret(key, key_option, stage, input, format, verbosity, global_scope, p
 def list_secret(stage, uninherited, decrypt, filter, query_all, query, format, verbosity, global_scope, project_scope, config_override, working_dir):
 
     def validate_command_usage():
-        nonlocal stage, query
-        stage = Stages.normalize(stage)
+        nonlocal query
         defaultQuery = '{Secrets: Secrets[*].{Key: Key, Value: Value, Scope: Scope, Origin: Origin}}'
         query = validate_query_argument(query, query_all, defaultQuery)
         validate_not_all_provided([global_scope, project_scope], ["'--global-scope'", "'--project-scope'"])
@@ -493,8 +522,8 @@ def list_secret(stage, uninherited, decrypt, filter, query_all, query, format, v
 @click.argument('key', required=False)
 @click.option('-k', '--key', 'key_option', required=False, metavar='<key>', help=f"{CLI_PARAMETERS_HELP['parameter']['key']}")
 @click.option('-s', '--stage', metavar='<name>[/<number>]', default='default', help=f"{CLI_PARAMETERS_HELP['common']['stage']}")
-@click.option('--revision', required=False, help=f"{CLI_PARAMETERS_HELP['secret']['revision']}")
-@click.option('--history', required=False, is_flag=True, help=f"{CLI_PARAMETERS_HELP['secret']['history']}")
+@click.option('--revision', required=False, help=f"{CLI_PARAMETERS_HELP['parameter']['revision']}")
+@click.option('-d', '--decrypt', required=False, is_flag=True, default=False, show_default=True, help=f"{CLI_PARAMETERS_HELP['secret']['decrypt']}")
 @click.option('-a', '--query-all', required=False, is_flag=True, default=False, show_default=True, help=f"{CLI_PARAMETERS_HELP['common']['query_all']}")
 @click.option('-q', '--query', metavar='<jmespath>', required=False, help=f"{CLI_PARAMETERS_HELP['common']['query']}")
 @click.option('-f', '--format', required=False, type=click.Choice(['json', 'yaml', 'raw', 'csv']), default='json', show_default=True, help=f"{CLI_PARAMETERS_HELP['common']['format']}")
@@ -503,18 +532,13 @@ def list_secret(stage, uninherited, decrypt, filter, query_all, query, format, v
 @click.option('--project-scope', required=False, is_flag=True, help=f"{CLI_PARAMETERS_HELP['common']['project_scope']}")
 @click.option('--config-override', 'config_override', metavar='<key>=<value>,...', required=False, help=f"{CLI_PARAMETERS_HELP['common']['config']}")
 @click.option('-w','--working-dir', metavar='<path>', type=click.Path(exists=True, file_okay=False), required=False, help=f"{CLI_PARAMETERS_HELP['common']['working_dir']}")
-def get_secret(key, key_option, stage, revision, history, query_all, query, format, verbosity, global_scope, project_scope, config_override, working_dir):
+def get_secret(key, key_option, stage, revision, decrypt, query_all, query, format, verbosity, global_scope, project_scope, config_override, working_dir):
 
     def validate_command_usage():
-        nonlocal stage, key, query
-        stage = Stages.normalize(stage)
+        nonlocal key, query
         validate_not_all_provided([key, key_option], ["KEY", "'-k' / '--key'"])
         key = validate_at_least_one_provided([key, key_option], ["KEY", "'-k' / '--key'"])
-        validate_not_all_provided([history, revision], ["'--history'", "'--revision'"])
-        if history:
-            defaultQuery = '{Revisions: Revisions[*].{RevisionId: RevisionId, Date: Date, Value: Value, Scope: Scope, Origin: Origin}}'
-        else:
-            defaultQuery = '{Value: Value}'
+        defaultQuery = '{Value: Value}'
         query = validate_query_argument(query, query_all, defaultQuery)
         validate_not_all_provided([global_scope, project_scope], ["'--global-scope'", "'--project-scope'"])
 
@@ -526,10 +550,53 @@ def get_secret(key, key_option, stage, revision, history, query_all, query, form
                         'global' if global_scope else 'project' if project_scope else 'application',
                         config_override)
 
-        if history:
-            result = Secrets.history(stage, key)
-        else:
-            result = Secrets.get(stage, key, revision)
+        result = Secrets.get(stage, key, revision, decrypt)
+        output = format_data(result, query, format)
+        if output: Pager.page(output)
+
+    except DSOException as e:
+        Logger.error(e.message)
+    except Exception as e:
+        msg = getattr(e, 'message', getattr(e, 'msg', str(e)))
+        Logger.critical(msg)
+        if verbosity >= log_levels['full']:
+            raise
+
+###--------------------------------------------------------------------------------------------
+
+@secret.command('history', context_settings=DEFAULT_CLICK_CONTEXT, short_help=f"{CLI_COMMANDS_SHORT_HELP['secret']['history']}")
+@command_doc(CLI_COMMANDS_HELP['secret']['history'])
+@click.argument('key', required=False)
+@click.option('-k', '--key', 'key_option', required=False, metavar='<key>', help=f"{CLI_PARAMETERS_HELP['parameter']['key']}")
+@click.option('-s', '--stage', metavar='<name>[/<number>]', default='default', help=f"{CLI_PARAMETERS_HELP['common']['stage']}")
+@click.option('-d', '--decrypt', required=False, is_flag=True, default=False, show_default=True, help=f"{CLI_PARAMETERS_HELP['secret']['decrypt']}")
+@click.option('-a', '--query-all', required=False, is_flag=True, default=False, show_default=True, help=f"{CLI_PARAMETERS_HELP['common']['query_all']}")
+@click.option('-q', '--query', metavar='<jmespath>', required=False, help=f"{CLI_PARAMETERS_HELP['common']['query']}")
+@click.option('-f', '--format', required=False, type=click.Choice(['json', 'yaml', 'raw', 'csv']), default='json', show_default=True, help=f"{CLI_PARAMETERS_HELP['common']['format']}")
+@click.option('-b', '--verbosity', metavar='<number>', required=False, type=RangeParamType(click.INT, minimum=0, maximum=5), default='2', show_default=True, help=f"{CLI_PARAMETERS_HELP['common']['verbosity']}")
+@click.option('--global-scope', required=False, is_flag=True, help=f"{CLI_PARAMETERS_HELP['common']['global_scope']}")
+@click.option('--project-scope', required=False, is_flag=True, help=f"{CLI_PARAMETERS_HELP['common']['project_scope']}")
+@click.option('--config-override', 'config_override', metavar='<key>=<value>,...', required=False, help=f"{CLI_PARAMETERS_HELP['common']['config']}")
+@click.option('-w','--working-dir', metavar='<path>', type=click.Path(exists=True, file_okay=False), required=False, help=f"{CLI_PARAMETERS_HELP['common']['working_dir']}")
+def history_secret(key, key_option, stage, decrypt, query_all, query, format, verbosity, global_scope, project_scope, config_override, working_dir):
+
+    def validate_command_usage():
+        nonlocal key, query
+        validate_not_all_provided([key, key_option], ["KEY", "'-k' / '--key'"])
+        key = validate_at_least_one_provided([key, key_option], ["KEY", "'-k' / '--key'"])
+        defaultQuery = '{Revisions: Revisions[*].{RevisionId: RevisionId, Date: Date, Value: Value}}'
+        query = validate_query_argument(query, query_all, defaultQuery)
+        validate_not_all_provided([global_scope, project_scope], ["'--global-scope'", "'--project-scope'"])
+
+
+    try:
+        Logger.set_verbosity(verbosity)
+        validate_command_usage()
+        Configs.load(working_dir if working_dir else os.getcwd(),
+                        'global' if global_scope else 'project' if project_scope else 'application',
+                        config_override)
+
+        result = Secrets.history(stage, key, decrypt)
         output = format_data(result, query, format)
         if output: Pager.page(output)
 
@@ -560,8 +627,7 @@ def delete_secret(key, key_option, input, format, stage, verbosity, global_scope
     secrets = []
 
     def validate_command_usage():
-        nonlocal stage, key, secrets
-        stage = Stages.normalize(stage)
+        nonlocal key, secrets
         validate_not_all_provided([global_scope, project_scope], ["'--global-scope'", "'--project-scope'"])
 
         if input:
@@ -626,8 +692,7 @@ def delete_secret(key, key_option, input, format, stage, verbosity, global_scope
 def list_template(uninherited, stage, include_contents, filter, query_all, query, format, verbosity, global_scope, project_scope, config_override, working_dir):
 
     def validate_command_usage():
-        nonlocal stage, query
-        stage = Stages.normalize(stage)
+        nonlocal query
         if include_contents:
             defaultQuery = '{Templates: Templates[*].{Key: Key, Scope: Scope, Origin: Origin, RenderTo: RenderTo, Contents: Contents}}'
         else:
@@ -730,8 +795,7 @@ def add_template(key, key_option, render_path, input, recursive, stage, verbosit
         return result
 
     def validate_command_usage():
-        nonlocal stage, input, key, templates
-        stage = Stages.normalize(stage)
+        nonlocal input, key, templates
         key = validate_not_all_provided([key, key_option], ["KEY", "'-k' / '--key'"])
         validate_not_all_provided([global_scope, project_scope], ["'--global-scope'", "'--project-scope'"])
 
@@ -798,7 +862,6 @@ def add_template(key, key_option, render_path, input, recursive, stage, verbosit
 @click.option('-k', '--key', 'key_option', metavar='<key>', required=False, help=f"{CLI_PARAMETERS_HELP['template']['key']}")
 @click.option('-s', '--stage', metavar='<name>[/<number>]', default='default', help=f"{CLI_PARAMETERS_HELP['common']['stage']}")
 @click.option('--revision', required=False, help=f"{CLI_PARAMETERS_HELP['parameter']['revision']}")
-@click.option('--history', required=False, is_flag=True, help=f"{CLI_PARAMETERS_HELP['parameter']['history']}")
 @click.option('--include-contents', required=False, is_flag=True, help=f"{CLI_PARAMETERS_HELP['template']['include_contents']}")
 @click.option('-a', '--query-all', required=False, is_flag=True, default=False, show_default=True, help=f"{CLI_PARAMETERS_HELP['common']['query_all']}")
 @click.option('-q', '--query', metavar='<jmespath>', required=False, help=f"{CLI_PARAMETERS_HELP['common']['query']}")
@@ -808,22 +871,13 @@ def add_template(key, key_option, render_path, input, recursive, stage, verbosit
 @click.option('--project-scope', required=False, is_flag=True, help=f"{CLI_PARAMETERS_HELP['common']['project_scope']}")
 @click.option('--config-override', 'config_override', metavar='<key>=<value>,...', required=False, help=f"{CLI_PARAMETERS_HELP['common']['config']}")
 @click.option('-w','--working-dir', metavar='<path>', type=click.Path(exists=True, file_okay=False), required=False, help=f"{CLI_PARAMETERS_HELP['common']['working_dir']}")
-def get_template(key, key_option, stage, revision, history, include_contents, query_all, query, format, verbosity, global_scope, project_scope, config_override, working_dir):
+def get_template(key, key_option, stage, revision, include_contents, query_all, query, format, verbosity, global_scope, project_scope, config_override, working_dir):
 
     def validate_command_usage():
-        nonlocal stage, key, query
-        stage = Stages.normalize(stage)
+        nonlocal key, query
         validate_not_all_provided([key, key_option], ["KEY", "'-k' / '--key'"])
         key = validate_at_least_one_provided([key, key_option], ["KEY", "'-k' / '--key'"])
-        validate_not_all_provided([history, revision], ["'--history'", "'--revision'"])
-        # validate_not_all_provided([history, revision], ["'--history'", "'--include-contents'"])
-        if history:
-            if include_contents:
-                defaultQuery = '{Revisions: Revisions[*].{RevisionId: RevisionId, Date: Date, Scope: Scope, Origin: Origin, Contents: Contents}}'
-            else:
-                defaultQuery = '{Revisions: Revisions[*].{RevisionId: RevisionId, Date: Date, Scope: Scope, Origin: Origin}}'
-        else:
-            defaultQuery = '{Contents: Contents}'
+        defaultQuery = '{Contents: Contents}'
         query = validate_query_argument(query, query_all, defaultQuery)
         validate_not_all_provided([global_scope, project_scope], ["'--global-scope'", "'--project-scope'"])
 
@@ -834,10 +888,54 @@ def get_template(key, key_option, stage, revision, history, include_contents, qu
                         'global' if global_scope else 'project' if project_scope else 'application',
                         config_override)
 
-        if history:
-            result = Templates.history(stage, key, include_contents)
+        result = Templates.get(stage, key, revision)
+        output = format_data(result, query, format)
+        if output: Pager.page(output)
+    except DSOException as e:
+        Logger.error(e.message)
+    except Exception as e:
+        msg = getattr(e, 'message', getattr(e, 'msg', str(e)))
+        Logger.critical(msg)
+        if verbosity >= log_levels['full']:
+            raise
+
+###--------------------------------------------------------------------------------------------
+
+@template.command('history', context_settings=DEFAULT_CLICK_CONTEXT, short_help=f"{CLI_COMMANDS_SHORT_HELP['template']['history']}")
+@command_doc(CLI_COMMANDS_HELP['template']['history'])
+@click.argument('key', required=False)
+@click.option('-k', '--key', 'key_option', metavar='<key>', required=False, help=f"{CLI_PARAMETERS_HELP['template']['key']}")
+@click.option('-s', '--stage', metavar='<name>[/<number>]', default='default', help=f"{CLI_PARAMETERS_HELP['common']['stage']}")
+@click.option('--include-contents', required=False, is_flag=True, help=f"{CLI_PARAMETERS_HELP['template']['include_contents']}")
+@click.option('-a', '--query-all', required=False, is_flag=True, default=False, show_default=True, help=f"{CLI_PARAMETERS_HELP['common']['query_all']}")
+@click.option('-q', '--query', metavar='<jmespath>', required=False, help=f"{CLI_PARAMETERS_HELP['common']['query']}")
+@click.option('-f', '--format', required=False, type=click.Choice(['json', 'yaml', 'raw', 'csv']), default='json', show_default=True, help=f"{CLI_PARAMETERS_HELP['common']['format']}")
+@click.option('-b', '--verbosity', metavar='<number>', required=False, type=RangeParamType(click.INT, minimum=0, maximum=5), default='2', show_default=True, help=f"{CLI_PARAMETERS_HELP['common']['verbosity']}")
+@click.option('--global-scope', required=False, is_flag=True, help=f"{CLI_PARAMETERS_HELP['common']['global_scope']}")
+@click.option('--project-scope', required=False, is_flag=True, help=f"{CLI_PARAMETERS_HELP['common']['project_scope']}")
+@click.option('--config-override', 'config_override', metavar='<key>=<value>,...', required=False, help=f"{CLI_PARAMETERS_HELP['common']['config']}")
+@click.option('-w','--working-dir', metavar='<path>', type=click.Path(exists=True, file_okay=False), required=False, help=f"{CLI_PARAMETERS_HELP['common']['working_dir']}")
+def history_template(key, key_option, stage, include_contents, query_all, query, format, verbosity, global_scope, project_scope, config_override, working_dir):
+
+    def validate_command_usage():
+        nonlocal key, query
+        validate_not_all_provided([key, key_option], ["KEY", "'-k' / '--key'"])
+        key = validate_at_least_one_provided([key, key_option], ["KEY", "'-k' / '--key'"])
+        if include_contents:
+            defaultQuery = '{Revisions: Revisions[*].{RevisionId: RevisionId, Date: Date, Contents: Contents}}'
         else:
-            result = Templates.get(stage, key, revision)
+            defaultQuery = '{Revisions: Revisions[*].{RevisionId: RevisionId, Date: Date}}'
+        query = validate_query_argument(query, query_all, defaultQuery)
+        validate_not_all_provided([global_scope, project_scope], ["'--global-scope'", "'--project-scope'"])
+
+    try:
+        Logger.set_verbosity(verbosity)
+        validate_command_usage()
+        Configs.load(working_dir if working_dir else os.getcwd(),
+                        'global' if global_scope else 'project' if project_scope else 'application',
+                        config_override)
+
+        result = Templates.history(stage, key, include_contents)
         output = format_data(result, query, format)
         if output: Pager.page(output)
     except DSOException as e:
@@ -867,8 +965,7 @@ def delete_template(key, key_option, stage, input, format, verbosity, global_sco
     templates = []
 
     def validate_command_usage():
-        nonlocal key, stage, templates
-        stage = Stages.normalize(stage)
+        nonlocal key, templates
         validate_not_all_provided([global_scope, project_scope], ["'--global-scope'", "'--project-scope'"])
 
         if input:
@@ -924,8 +1021,6 @@ def delete_template(key, key_option, stage, input, format, verbosity, global_sco
 def render_template(stage, filter, verbosity, global_scope, project_scope, config_override, working_dir):
 
     def validate_command_usage():
-        nonlocal stage
-        stage = Stages.normalize(stage)
         validate_not_all_provided([global_scope, project_scope], ["'--global-scope'", "'--project-scope'"])
         if filter:
             try:
@@ -1438,9 +1533,9 @@ def delete_config(key, key_option, global_, verbosity, global_scope, project_sco
 
 #     try:
 #         Logger.set_verbosity(verbosity)
-        Configs.load(working_dir if working_dir else os.getcwd(),
-                        'global' if global_scope else 'project' if project_scope else 'application',
-                        config_override)
+        # Configs.load(working_dir if working_dir else os.getcwd(),
+        #                 'global' if global_scope else 'project' if project_scope else 'application',
+        #                 config_override)
 #         validate_command_usage()
 
 
