@@ -53,7 +53,6 @@ class AwsSsmParameterProvider(ParameterProvider):
                                     'Origin': details['Origin'],
                                     'RevisionId': str(details['Version']),
                                     'Date': details['Date'],
-                                    # 'Version': details['Version'],
                                     'Path': details['Path'],
                                     })
 
@@ -81,8 +80,7 @@ class AwsSsmParameterProvider(ParameterProvider):
                     'Stage': Stages.shorten(stage),
                 },
                 'RevisionId': str(response['Version']),
-                # 'Version': response['Version'],
-                'Path': path,
+                # 'Path': path,
                 }
 
 ###--------------------------------------------------------------------------------------------
@@ -91,7 +89,7 @@ class AwsSsmParameterProvider(ParameterProvider):
         Logger.debug(f"Locating SSM parameter: project={project}, application={application}, stage={stage}, key={key}")
         found = locate_ssm_parameter_in_context_hierachy(project, application, stage, key, prefix=self.get_prefix())
         if not found:
-            raise DSOException(f"Parameter '{key}' not found in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
+            raise DSOException(f"Parameter '{key}' not found nor inherited in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
         else:
             if not found['Type'] == 'String':
                 raise DSOException(f"Parameter '{key}' not found in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
@@ -107,9 +105,8 @@ class AwsSsmParameterProvider(ParameterProvider):
                     'Value': parameters[0]['Value'],
                     'Scope': found['Scope'],
                     'Origin': found['Origin'],
-                    'User': parameters[0]['LastModifiedUser'],
-                    # 'Version': parameters[0]['Version'],
                     'Path': found['Name'],
+                    'User': parameters[0]['LastModifiedUser'],
                     }
         else:
             ### get specific revision
@@ -123,9 +120,8 @@ class AwsSsmParameterProvider(ParameterProvider):
                     'Value': parameters[0]['Value'],
                     'Scope': found['Scope'],
                     'Origin': found['Origin'],
-                    'User': parameters[0]['LastModifiedUser'],
-                    # 'Version': parameters[0]['Version'],
                     'Path': found['Name'],
+                    'User': parameters[0]['LastModifiedUser'],
                     }
 
         return result
@@ -147,13 +143,12 @@ class AwsSsmParameterProvider(ParameterProvider):
             [{
                 'RevisionId': str(parameter['Version']),
                 'Date': parameter['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
-                'Key': key,
+                # 'Key': key,
                 'Value': parameter['Value'],
-                'Scope': found['Scope'],
-                'Origin': found['Origin'],
+                # 'Scope': found['Scope'],
+                # 'Origin': found['Origin'],
                 'User': parameter['LastModifiedUser'],
-                # 'Version': parameter['Version'],
-                'Path': found['Name'],
+                # 'Path': found['Name'],
             } for parameter in parameters]
         }
 
@@ -178,7 +173,7 @@ class AwsSsmParameterProvider(ParameterProvider):
                 'Key': key, 
                 'Scope': found['Scope'],
                 'Origin': found['Origin'],
-                'Path': found['Name'],
+                # 'Path': found['Name'],
                 }
 
 ###--------------------------------------------------------------------------------------------
