@@ -1,3 +1,3 @@
-__package__ = 'mabledsocli.provider.secret.aws.ssm.v1'
+__package__ = 'dsocli.provider.secret.aws.ssm.v1'
 from .main import get_default_spec
 from .main import AwsSsmSecretProvider
