@@ -81,21 +81,21 @@ class AwsSsmSecretProvider(SecretProvider):
                     'Stage': Stages.shorten(stage),
                 },
                 'RevisionId': str(response['Version']),
-                'Path': path,
+                # 'Path': path,
                 }
 
 ###--------------------------------------------------------------------------------------------
 
-    def get(self, project, application, stage, key, revision=None):
+    def get(self, project, application, stage, key, revision=None, decrypt=False):
         Logger.debug(f"Locating SSM secret: project={project}, application={application}, stage={stage}, key={key}")
         found = locate_ssm_parameter_in_context_hierachy(project, application, stage, key, prefix=self.get_prefix())
         if not found:
-            raise DSOException(f"Secret '{key}' not found in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
+            raise DSOException(f"Secret '{key}' not found nor inherited in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
         else:
             if not found['Type'] == 'SecureString':
                 raise DSOException(f"Parameter '{key}' not found in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
         Logger.info(f"Getting SSM secret: path={found['Name']}")
-        response = ssm.get_parameter_history(Name=found['Name'], WithDecryption=True)
+        response = ssm.get_parameter_history(Name=found['Name'], WithDecryption=decrypt)
         secrets = sorted(response['Parameters'], key=lambda x: x['Version'], reverse=True)
         if revision is None:
             ### get the latest revision
@@ -106,8 +106,8 @@ class AwsSsmSecretProvider(SecretProvider):
                     'Value': secrets[0]['Value'],
                     'Scope': found['Scope'],
                     'Origin': found['Origin'],
-                    'User': secrets[0]['LastModifiedUser'],
                     'Path': found['Name'],
+                    'User': secrets[0]['LastModifiedUser'],
                     }
         else:
             ### get specific revision
@@ -121,15 +121,15 @@ class AwsSsmSecretProvider(SecretProvider):
                     'Value': secrets[0]['Value'],
                     'Scope': found['Scope'],
                     'Origin': found['Origin'],
-                    'User': secrets[0]['LastModifiedUser'],
                     'Path': found['Name'],
+                    'User': secrets[0]['LastModifiedUser'],
                     }
 
         return result
 
 ###--------------------------------------------------------------------------------------------
 
-    def history(self, project, application, stage, key):
+    def history(self, project, application, stage, key, decrypt=False):
         found = locate_ssm_parameter_in_context_hierachy(project, application, stage, key, prefix=self.get_prefix())
         if not found:
             raise DSOException(f"Secret '{key}' not found in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
@@ -137,18 +137,18 @@ class AwsSsmSecretProvider(SecretProvider):
             if not found['Type'] == 'SecureString':
                 raise DSOException(f"Parameter '{key}' not found in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
         Logger.info(f"Getting SSM secret: path={found['Name']}")
-        response = ssm.get_parameter_history(Name=found['Name'], WithDecryption=True)
+        response = ssm.get_parameter_history(Name=found['Name'], WithDecryption=decrypt)
         secrets = sorted(response['Parameters'], key=lambda x: x['Version'], reverse=True)
         result = { "Revisions":
             [{
                 'RevisionId': str(secret['Version']),
                 'Date': secret['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
-                'Key': key,
+                # 'Key': key,
                 'Value': secret['Value'],
-                'Scope': found['Scope'],
-                'Origin': found['Origin'],
+                # 'Scope': found['Scope'],
+                # 'Origin': found['Origin'],
                 'User': secret['LastModifiedUser'],
-                'Path': found['Name'],
+                # 'Path': found['Name'],
             } for secret in secrets]
         }
 
@@ -173,7 +173,7 @@ class AwsSsmSecretProvider(SecretProvider):
                 'Key': key, 
                 'Scope': found['Scope'],
                 'Origin': found['Origin'],
-                'Path': found['Name'],
+                # 'Path': found['Name'],
                 }
 
 ###--------------------------------------------------------------------------------------------
