@@ -80,7 +80,7 @@ class AwsSsmTemplateProvider(TemplateProvider):
 
 ###--------------------------------------------------------------------------------------------
 
-    def add(self, project, application, stage, key, contents):
+    def add(self, project, application, stage, key, contents, render_path=None):
         if len(contents) > 4096:
             raise DSOException(f"This template provider does not support templates larger than 4KB.")
         if not Stages.is_default(stage) and not Configs.allow_stage_templates:
@@ -103,7 +103,7 @@ class AwsSsmTemplateProvider(TemplateProvider):
                     'Stage': Stages.shorten(stage),
                 },
                 'RevisionId': str(response['Version']),
-                'Path': path,
+                # 'Path': path,
             }
 
 ###--------------------------------------------------------------------------------------------
@@ -112,7 +112,7 @@ class AwsSsmTemplateProvider(TemplateProvider):
         Logger.debug(f"Locating SSM template: project={project}, application={application}, stage={stage}, key={key}")
         found = locate_ssm_parameter_in_context_hierachy(project, application, stage, key, prefix=self.get_prefix())
         if not found:
-            raise DSOException(f"Template '{key}' not found in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
+            raise DSOException(f"Template '{key}' not found nor inherited in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
         else:
             if not found['Type'] == 'StringList':
                 raise DSOException(f"Template '{key}' not found in the given context: project={project}, application={application}, stage={Stages.shorten(stage)}")
@@ -151,7 +151,7 @@ class AwsSsmTemplateProvider(TemplateProvider):
 
 ###--------------------------------------------------------------------------------------------
 
-    def history(self, project, application, stage, key, include_contents):
+    def history(self, project, application, stage, key, include_contents=False):
         Logger.debug(f"Locating SSM template: project={project}, application={application}, stage={stage}, key={key}")
         found = locate_ssm_parameter_in_context_hierachy(project, application, stage, key, prefix=self.get_prefix())
         if not found:
@@ -167,11 +167,11 @@ class AwsSsmTemplateProvider(TemplateProvider):
                 [{
                     'RevisionId': str(template['Version']),
                     'Date': template['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
-                    'Key': key,
-                    'Scope': found['Scope'],
-                    'Origin': found['Origin'],
+                    # 'Key': key,
+                    # 'Scope': found['Scope'],
+                    # 'Origin': found['Origin'],
                     'User': template['LastModifiedUser'],
-                    'Path': found['Name'],
+                    # 'Path': found['Name'],
                     'Contents': self.unescape_curly_brackets(template['Value']),
                 } for template in parameters]
             }
@@ -180,11 +180,11 @@ class AwsSsmTemplateProvider(TemplateProvider):
                 [{
                     'RevisionId': str(template['Version']),
                     'Date': template['LastModifiedDate'].strftime('%Y/%m/%d-%H:%M:%S'),
-                    'Key': key,
-                    'Scope': found['Scope'],
-                    'Origin': found['Origin'],
+                    # 'Key': key,
+                    # 'Scope': found['Scope'],
+                    # 'Origin': found['Origin'],
                     'User': template['LastModifiedUser'],
-                    'Path': found['Name'],
+                    # 'Path': found['Name'],
                 } for template in parameters]
             }
 
@@ -209,7 +209,7 @@ class AwsSsmTemplateProvider(TemplateProvider):
                 'Key': key, 
                 'Scope': found['Scope'],
                 'Origin': found['Origin'],
-                'Path': found['Name'],
+                # 'Path': found['Name'],
                 }
 
 ###--------------------------------------------------------------------------------------------
