@@ -14,15 +14,15 @@ from .exceptions import DSOException
 from .stages import Stages
 
 class TemplateProvider(StoreProvider):
-    def list(self, project, application, stage, uninherited):
+    def list(self, project, application, stage, uninherited=False, include_contents=False, filter=None):
         raise NotImplementedError()
-    def add(self, project, application, stage, key, contents):
-        raise NotImplementedError()
-    def delete(self, project, application, stage, key):
+    def add(self, project, application, stage, key, contents, render_path=None):
         raise NotImplementedError()
     def get(self, project, application, stage, key, revision=None):
         raise NotImplementedError()
-    def history(self, project, application, stage, key):
+    def history(self, project, application, stage, key, include_contents=False):
+        raise NotImplementedError()
+    def delete(self, project, application, stage, key):
         raise NotImplementedError()
 
 
@@ -48,6 +48,7 @@ class TemplateManager():
     def list(self, stage, uninherited=False, include_contents=False, filter=None):
         project = Configs.project
         application = Configs.application
+        stage = Stages.normalize(stage)
         provider = Providers.TemplateProvider()
         renderPaths = Configs.get_template_render_paths()
         Logger.info(f"Start listing templates: project={project}, application={application}, stage={Stages.shorten(stage)}")
@@ -56,8 +57,6 @@ class TemplateManager():
             key = template['Key']
             if key in renderPaths:
                 renderPath = renderPaths[key]
-                # if not (renderPath == '.' or renderPath.startswith(f'.{os.sep}')):
-                #     renderPath = os.path.join('./', renderPath)
             else:
                 renderPath = f'.{os.sep}' + os.path.relpath(os.path.join(Templates.default_render_path, key), Configs.working_dir) 
 
@@ -69,26 +68,9 @@ class TemplateManager():
         self.validate_key(key)
         project = Configs.project
         application = Configs.application
+        stage = Stages.normalize(stage)
         provider = Providers.TemplateProvider()
         Logger.info(f"Start adding template: project={project}, application={application}, stage={Stages.shorten(stage)}, render_path={render_path}")
-        # if '..' in render_path:
-        #     raise DSOException(MESSAGES['InvalidRenderPath'].format(render_path))
-        # if render_path == '.' or render_path == f".{os.sep}":
-        #     render_path = os.path.relpath(os.path.join(self.default_render_path, key), Configs.working_dir)
-        # else:
-        #     if '**' in render_path:
-        #         render_path = render_path.replace('**', os.path.dirname(key))
-        #     if '*' in render_path:
-        #         render_path = render_path.replace('*', os.path.basename(key))
-        # if not re.match('^[A-Za-z0-9._/$-]+$', render_path):
-        #     raise DSOException(MESSAGES['InvalidRenderPath'].format(render_path))
-        # if os.path.isdir(render_path):
-        #     raise DSOException(MESSAGES['InvalidRenderPathExistingDir'].format(render_path))
-        # if os.path.isabs(render_path):
-        #     Logger.warn(f"Render path {render_path} is not releative to the application root.")
-        # else:
-        #     if not (render_path == '.' or render_path.startswith(f".{os.sep}")):
-        #         render_path = os.path.join(f".{os.sep}", render_path)
         result = provider.add(project, application, stage, key, contents)
         result['RenderTo'] = render_path
         if os.path.abspath(render_path) == os.path.abspath(os.path.join(self.default_render_path, key)):
@@ -101,6 +83,7 @@ class TemplateManager():
         # self.validate_key(key)
         project = Configs.project
         application = Configs.application
+        stage = Stages.normalize(stage)
         provider = Providers.TemplateProvider()
         Logger.info(f"Start getting the contents of template '{key}': project={project}, application={application}, stage={Stages.shorten(stage)}")
         return provider.get(project, application, stage, key, revision)
@@ -109,6 +92,7 @@ class TemplateManager():
         # self.validate_key(key)
         project = Configs.project
         application = Configs.application
+        stage = Stages.normalize(stage)
         provider = Providers.TemplateProvider()
         Logger.info(f"Start getting the history of template '{key}': project={project}, application={application}, stage={Stages.shorten(stage)}")
         return provider.history(project, application, stage, key, include_contents)
@@ -117,6 +101,7 @@ class TemplateManager():
         # self.validate_key(key)
         project = Configs.project
         application = Configs.application
+        stage = Stages.normalize(stage)
         provider = Providers.TemplateProvider()
         Logger.info(f"Start deleting template '{key}': project={project}, application={application}, stage={Stages.shorten(stage)}")
         result = provider.delete(project, application, stage, key)
@@ -126,6 +111,7 @@ class TemplateManager():
     def render(self, stage, filter=None):
         project = Configs.project
         application = Configs.application
+        stage = Stages.normalize(stage)
         Logger.info(f"Start rendering templates: project={project}, application={application}, stage={Stages.shorten(stage)}")
 
         if Stages.is_default(stage):
@@ -140,7 +126,7 @@ class TemplateManager():
         Logger.info("Loading DSO meta vars...")
         meta = {'dso': {'stage': Stages.shorten(stage), 'config': Configs.merged_config}}
 
-        Logger.info("Merging parameters...")
+        Logger.info("Merging all parameters...")
         merged = deflatten_dict({x['Key']: x['Value'] for x in parameters['Parameters']})
         merge_dicts(deflatten_dict({x['Key']: x['Value'] for x in secrets['Secrets']}), merged)
         merge_dicts(meta, merged)
